package info.markhillman.ImageViewer;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import info.markhillman.mwindow.*;

public class ImageViewer {

    public static void main(String[] args) {

        String imagePath;
        MDragWindow window;
        MScrollPane scrollPane;
        ImagePanel imagePanel;

        //Get the image to load from the command line strings
        if (args.length > 0) {

            //The first arguement is the filepath
            imagePath = args[0];

            //Try and create the new ImagePanel
            try {
                //Create a new JFrame
                File file = new File(imagePath);
                window = new MDragWindow("ImageViewer - " + file.getName());
                imagePanel = new ImagePanel(imagePath);
            }
            catch (IOException e) {
                e.printStackTrace();
                System.out.println(imagePath + " could not be found!");
                return;
            }

            if (imagePanel != null) {

                //Check the size of the image
                int width, height;
                boolean adjustHeight = false, adjustWidth = false;
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                width = (int)screenSize.getWidth() - 100;
                height = (int)screenSize.getHeight() - 100;

                //Width check
                if (imagePanel.getImage().getWidth() < width)
                    width = imagePanel.getImage().getWidth();
                else
                    adjustHeight = true;

                //Height check
                if (imagePanel.getImage().getHeight() < height)
                    height = imagePanel.getImage().getHeight() + window.getTitleBarHeight();
                else
                    adjustWidth = true;

                //Adjust scrollbars
                if (adjustHeight)
                    height += 17;
                if (adjustWidth)
                    width += 17;

                //Create the scroll pane for the image
                scrollPane = new MScrollPane(imagePanel);
                scrollPane.getVerticalScrollBar().setUnitIncrement(16);
                scrollPane.getHorizontalScrollBar().setUnitIncrement(16);
                scrollPane.setBorder(null);

                //Add everything to the window and display it
                window.add(scrollPane);
                window.setSize(new Dimension(width, height));
                window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
                window.setResizable(true);
                window.setVisible(true);
                window.revalidate();
            }
        }
    }
}
